/*
 * ========================================================================
 * (c) Copyright 2021 Micro Focus or one of its affiliates.
 * ========================================================================
 */

var getOpText = function(op){
    if (op != null){
    //Not all operators are built the same so this just expands on the smaller ones.
        switch(op){
            case "IN": return "EQUALS_ONE_OF";
            case 'GT': return 'GREATER_THAN';
            case 'LT': return 'LESS_THAN';
            case 'EQ': return 'EQUAL_TO';
            case 'GE': return 'GREATER_THAN_EQ';
            case 'LE': return 'LESS_THAN_EQ';
            case 'NE': return 'NOT_EQUAL';
            case 'ADD': return "+";
            case 'SUB': return "-";
            case 'MULT': return '*';
            case 'DIV': return '/';
            case 'MOD': return '%';
            default: return op;
        }
    }
};

var getAttributeText = function(attr){
    if (attr !== null){
        switch (attr){
            case "name": return "NAME";
            case "application": return "APPLICATION";
            case "cost": return "COST";
            case "permissionOwners": return "OWNERS";
            case "risk" : return "RISK";
            case "staticPermission": return "STATIC_PERMISSION";
            case "entityCategories": return "CATEGORIES";
            case "approvedBy": return "APPROVED_BY";
            case "changedBy": return "CHANGED_BY";
            case "createdBy": return "CREATED_BY";
            case "description": return "DESCRIPTION";
            case "gracePeriod": return "GRACE_PERIOD";
            case "owners": "OWNERS";
            case "managers": return "MANAGERS";
            case "fulfiller": return "FULFILLER";
            case "publishedBy": return "PUBLISHED_BY";
            case "membershipUpdatedby": return "MEMBERSHIP_UPDATED_BY";
            case "authsUpdatedBy": return "AUTHS_UPDATED_BY";
            
            case "displayName": return "DISPLAY_NAME";
            case "affiliatedId": return "AFFILIATED_ID";
            case "city":  return "CITY";
            case "country": return  "COUNTRY";
            case "createDate": return "CREATE_DATE";
            case "deleteDate": return "DELETE_DATE";
            case "department": return "DEPARTMENT";
            case "dn": return "LDAP";
            case "employeeType": return "EMPLOYEE_TYPE";
            case "jobCode": return "JOB_CODE";
            case "jobTitle": return "JOB_TITLE";
            case "location": return "LOCATION";
            case "originalCreateDate": return "ORIG_CREATE_DATE";
            case "state":   return "STATE";
            case "supervisor": return "SUPERVISOR";
            case "workforceId": return "WORKFORCE_ID";
            
            case "accountOwners": return "CUSTODIANS";
            case "classificationId": return "CLASSIFICATION";
            case "relationToUserType": return "ACC_ASSIGNMENT_TYPE";
            default: return attr;
        
        }
    }
}

  var evaluateExpression = function(expr){ 
    resultList = [];
    
    var operator = expr.value.operator;
    var count = 1;
    if (expr.value.operands !== undefined){
        var operands1 = expr.value.operands;
        operands1.forEach( function(operands){
                if (operands.type == "EXPRESSION"){
                    resultList.push("(");
                    resultList = resultList.concat(evaluateExpression(operands));
                    //We use the concat function here to add a list onto a list
                    //.push in this case kind of excludes some info.
                    resultList.push(")");
                }//Use recursion to put in another expression in parenthesis.
                 //This won't exactly mimic what's in IG, but it should return the same order of operations.
                 
                else if (operands.type == "FUNCTION"){
                    resultList.push(operands.value.funcType);
                } //Push the function type with the operator from earlier
                  
                else if (operands.type == "DECIMAL_NUMBER"){
                    resultList.push(operands.value);
                } //Just push the value of the number
                

                //In between iterations, push the operator.
                 //If it's the last value of operands, don't push it.

                if (count !== operands1.length){
                    resultList.push(getOpText(operator));
                }
                count += 1;
                
            }
        )
    }
    else if (expr.value !== undefined){
        resultList.push(expr.value);
    }
    
    return resultList;
  }
  
var parseEpoch = function (str)
  {
    var d = new Date(str);
    return d;
  }

var parseJson = function(jsonObj){
    var resultList = [];
    if (jsonObj !== null){
        if (jsonObj.childCritera !== null){
            if (jsonObj.operator == "NOT"){
                resultList.push("not (");
            }
            
            var children = jsonObj.childCriteria;
            var child_length = children.length;
            var count_children = 1;
            
            
            children.forEach(
                function(children){

                    if (children.values !== undefined || children.valueAttrKey !== undefined || children.value !== undefined || children.childCriteria !== undefined || children.entities !== undefined || children.operator == "IS_EMPTY" || children.expr !== undefined){
                    
                      if (count_children != 1){
                        jsonObj.operator == "NOT" ? resultList.push("OR") : resultList.push(jsonObj.operator);
                      }
                      
                     if (children.attributeKey !== undefined){
                          resultList.concat(getAttributeText(children.attributeEntity), ": ");
                        
                        if(children.attributeKey !== null){
                            resultList.push(getAttributeText(children.attributeKey));
                        }
                        
                        if(children.operator !== null){
                            resultList.push(getOpText(children.operator));
                        }
                        if((children.expr !== undefined) || children.attributeKey == "createDate" || children.attributeKey == "deleteDate" || children.attributeKey == "originalCreateDate") {
                            //Unfortunately extended date type attributes can't be differentiated 
                            if(children.value !== undefined) {
                                resultList.push(parseEpoch(children.value));
                            }
                            else if (children.expr !== undefined){
                                resultList = resultList.concat(evaluateExpression(children.expr));
                                //We use the concat function here to add a list onto a list
                                //.push won't work well here. It makes it ugly and kind of unreadable.
                            }
                        }
                        else if (children.values !== undefined){
                            for (var i = 0; i < children.values.length; i++){
                            //Goal is to have "[something1, something2, .... , Or somethingN]
                                if (i == 0 && children.values.length > 1){
                                    resultList.push("[");
                                }
                                if (i == children.values.length - 1 && children.values.length != 1){
                                    resultList.push("OR", children.values[i]); 
                                    //When testing in IG, only the OR operator would come out
                                }
                                else if (i > 0){
                                    resultList.push(", ", children.values[i]);
                                }
                                else{
                                    resultList.push(children.values[i]);
                                }
                                if (i == children.values.length-1 && children.values.length > 1){
                                    resultList.push("]");
                                }
                            }
                        }
                        else if (children.valueAttrKey !== undefined){
                        //This is for when selecting attribute instead of just value
                            resultList.push(getAttributeText(children.valueAttrKey));
                        }
                        else if (children.value !== undefined){
                        // This case if for just attribute "value"
                            resultList.push(children.value);
                        }
                        else if (children.entities !== undefined){
                        //Nested contextual conditions use entities for users
                         for (var i = 0; i < children.entities.length; i++){
                                if (i == 0 && children.entities.length > 1){
                                    resultList.push("[");
                                }
                                if (i == children.entities.length - 1 && children.entities.length != 1){
                                    resultList.push("OR", children.entities[i].displayName); 
                                    //When testing in IG, only the OR operator would come out
                                }
                                else if (i > 0){
                                    resultList.push(", ", children.entities[i].displayName);
                                }
                                else{
                                    resultList.push(children.entities[i].displayName);
                                }
                                if (i == children.entities.length-1 && children.entities.length > 1){
                                    resultList.push("]");
                                }
                            }
                          
                        }
                      
                    }
                    else if (children.attributeKey === undefined && children.childCriteria !== undefined){
                    //For nested criteria conditions
                        resultList.push("(");
                        resultList = resultList.concat(parseJson(children));
                        resultList.push(")");
                    }
                    if (count_children == child_length && jsonObj.operator == "NOT"){
                        resultList.push(")");
                    }
                    count_children += 1;
                  }
                  else{
                    child_length--;
                  }
                  
                 
                    
                  
                }
            )
        }
    }
    return resultList;
    
}


var localizeJSON = function(jsonStr, localizer){
    var jsonObj = JSON.parse(jsonStr);
    var resultList = [];
    
    if (jsonObj !== undefined && localizer !== undefined){
    
        resultList = resultList.concat(parseJson(jsonObj));
        
        return resultList.map(function(str) {
         return localizer.getString(str);
      }).join(' ');
    }
}